#include <allegro.h>
#include <ctime>
#include <math.h>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <vector>

using namespace std;

#ifdef __APPLE__
#include <ApplicationServices/ApplicationServices.h>
#endif

#include "musicback.h"
#include "timer.hpp"

const int resx = 640;
const int resy = 480;

const int screenwidth=80;
const int screenheight=60;

int mx;
int my;
int rmx;
int rmy;

bool freezecursor;
int shuddertimer=0;


string str;
int randx;
int randy;
int textmode;
int convline;

BITMAP* buffer;
BITMAP* bmpFaces;
BITMAP* bmpTable;
BITMAP* bmpDrill;
BITMAP* bmpDropper;
BITMAP* bmpHead;
BITMAP* bmpCar;
BITMAP* bmpMan;
BITMAP* bmpCursor;
BITMAP* bmpHead2;
BITMAP* bmpHoleMark;
BITMAP* bmpSaw;

bool comingfromintro=true;

int screenoffset;

const int MAPWIDTH=7429;

bool escpressed=false;
bool escdown=false;               

bool actionpush;
bool actiondown;
    
    
enum GameState {
    GS_Intro,
    GS_IntroBlackout,
    GS_Title,
    GS_TitleBlackout,
    GS_Prelude,
    GS_PreludeDialogue,
    GS_PostPrelude,
    GS_PostPrelude2,
    GS_Game,
    GS_Postlude,
    GS_LoopBack,
    GS_GameOver,
    GS_Quit
};

GameState gamestate;

int cursor;
int cursorframe;
float drillprogress;
float drillratio;
bool drillbone;
bool nodrill;
float drilldepth;
bool overdone;

struct Pos
{
    int x;
    int y;
};
vector<Pos> drops;

enum BrainType {
    BT_None=0, 
    BT_Drive=1,
    BT_SelfAwareness=2,
    BT_Desire=3,
    BT_Obedience=4,
    BT_HandEyeCoord=5,
    BT_Damage=5,
    BT_MAX=6,
};

struct Hole
{
    int x;
    int y;
    int depth;
    int acid;
    BrainType GetType() ;
    Hole()
    {
        x=0;
        y=0;
        acid=0;
    }
};


void wrapprint( const string& str, int x, int y, int width=19 )
{
	
    if (width==0)
        return;
	
    string s=str+" ";
    int cury=y;
	unsigned int startp=0;
    while (startp<s.size())
    {
        unsigned int endp=startp+1;
        int lastspace=-1;
		
        while (endp<=width+startp)
        {
			
            if (endp==s.size())
            {
                lastspace=endp;
                break;
            }
            if (s[endp]=='\n')
            {
                lastspace=endp;
                break;
            }
            else if (s[endp]==' ')
            {
                lastspace=endp;
            }
            endp++;
        }
		
        if (lastspace>=0)//wrap a word around
        {
            //if two consecutive newlines
            if (s.substr(startp,lastspace-startp)=="\n")
            {
                startp=lastspace+1;
            }
            else
            {
				textout_ex(buffer, font, s.substr(startp,lastspace-startp).c_str(), x, cury, makecol(255,255,255), -1);
                startp=lastspace+1;
            }
        }
        else //word bigger than line, just thunk
        {
			textout_ex(buffer, font, s.substr(startp,endp-startp).c_str(), x, cury, makecol(255,255,255), -1);
            startp+=width;
        }
		
        cury+=10;
		
    }
}

BrainType Hole::GetType() 
{   
    int headoverlaps=-1;
    if ( x>=29 && y >=29 && x<29+23 && y<29+27)
    {
        int pix = getpixel(bmpHead2, x-29, y-29);
        if (pix==makecol(255,0,0))
        {
            headoverlaps=0;
        }
        else if (pix==makecol(0,255,0))
        {
            headoverlaps=1;
        }
        else if (pix==makecol(0,0,255))
        {
            headoverlaps=2;
        }
        else if (pix==makecol(0,255,255))
        {
            headoverlaps=3;
        }
    }
    
    if (headoverlaps==-1)
    {
        return BT_None;
    }
    
    if (depth<4)
    {
        return BT_Drive;
    }
    else if (depth>=9)
    {
        return BT_Damage;
    }
    else
    {
        switch (headoverlaps)
        {
            case 0:
            return BT_SelfAwareness;
            break;
            case 1:
            return BT_Desire;
            break;
            case 2:
            return BT_Obedience;
            break;
            case 3:
            return BT_HandEyeCoord;
            break;
        }
    }
    
    //should never get here
    return BT_Damage;
}

vector<Hole> holes;

int nBrainState;
int endStates[BT_MAX];
bool bParalysedButSane;
int answers[10];

void calcBrainState()
{

    
    int nDamage=0;
    nBrainState=0;
    bParalysedButSane=false;
    for (int i=0;i<10;i++)
        answers[i]=-1;
    for (int i=0;i<BT_MAX;i++)
    {   
        endStates[i]=0;
    }
    
    for (int i=0;i<holes.size();i++)
    {
        nDamage++;
        if (holes[i].acid==1)
            nDamage++;
        else if (holes[i].acid>1)
            nDamage+=10;
        endStates[holes[i].GetType()]+=holes[i].acid;
            
        if  (holes[i].GetType()==BT_Damage)
        {
            nDamage++;        
        }
    }
    
    if (nDamage<2)
    {
    
            nBrainState=2;
    }
    else if (nDamage<10)
        nBrainState=1;
    else 
    {
        nBrainState=0;
        return;
    }
    
    
        
    //check nothing has greater than 1
    for (int i=BT_Drive;i<=BT_HandEyeCoord;i++)
    {
        if (endStates[i]>1)
        {
            nBrainState=0;
            return;
        }
    }
        
    //check for 'paralysed, but otherwise ok' ending
    bool bOk=true;
    for (int i=BT_Drive;i<BT_HandEyeCoord;i++)
    {
        if (endStates[i]>0)
        {
            bOk=false;
        }
    }
    if (bOk && endStates[BT_HandEyeCoord]==1)
    {
        bParalysedButSane=true;
        nBrainState=2;
        return;
    }
    else if (bOk)
    {
        return;//no more need to deal with state 2
    }
    
    if (endStates[BT_Drive]==0)
    {
        nBrainState=2;
        return;
    }
    
    //for the normal conv
                 //"you were asleep"
                 //"it's been over 24 hour since you've eaten"
                 //"are you hungry?
                 
    if (endStates[BT_SelfAwareness]==0)
    {
        answers[0]=0; //"yes"
                      //
    }
    else
    {
        answers[0]=1; //"I...I don't know."
                      //"You don't know?"
                      //"I don't know! I... I can't tell!"
                      //"That doesn't seem right
                      //"I guess we'll just have to try again
                      //"What...what do you mean?
    }
    if (endStates[BT_Desire]==0)//"Do you want some food?
    {
        answers[1]=0; //"Yes, please - I'm so hungry
                      //"That...that isn't right
                      //"What do you mean?
                      //"That isn't the right answer!
                      //"I don't understand
                      //"You should not desire to act on your drives
                      //"I'm sorry...I won't, just don't hurt me, please!
                      //"A man cannot recant his intentions!
                      //"They must be excised by another.
                      //"You are a failure...;
                      //"No...please no...
    }
    else
    {
        answers[1]=1; //"No
                      //"Good
    }                   
    if (endStates[BT_Obedience]==0)
    {                //"Here, eat this food
        answers[2]=0;//"I ... I don't want to
                     //"Hmm.  Why not?
                     //"Please, don't hurt me
                     //"Why won't you do as I say?!
                     //"Please, just let me go
                     //"YOU PIECE OF SHIT
                     //"Let me go, please!
                     //"I...this isn't working
    }
    else
    {
        answers[2]=1;//"Okay
                     //"There's a good chap
    }
    if (endStates[BT_HandEyeCoord]==0)
    {               //"There; I'll remove your restraints now
                    //"Thank you
                    //"Could you get me a glass of water from the kitchen
                    //"Okay...
                    
            answers[3]=0;
                    //"Thank you very much
                    //"I think...I think might be...perfect    
    }
    else
    {
            answers[3]=1;//"I...I can't move
                         //"You can't move?
                         //"Please...I can't move! 
                         //"That's unfortunate
                         //"What did you do to me?!
                         //"Don't worry, I'll end it for you
    }
    
 
}

void makehole(int x, int y, int depth)
{
    bool bFoundHole=false;
    int i;
    for (i=0;i<holes.size();i++)
    {
        if (holes[i].x==x && holes[i].y==y)
        {
            bFoundHole=true;
            break;
        }
    }
    if (!bFoundHole)
    {
        Hole h;
        h.x=x;
        h.y=y;
        h.depth=depth;
        holes.push_back(h);
    }
    else
    {
        holes[i].depth=max(holes[i].depth,depth);
    }
}

bool dropletCheck(int n) //must delete droplet if this returns true
{
    for (uint i=0;i<holes.size();i++)
    {
        if (drops[n].y==holes[i].y && abs(drops[n].x-holes[i].x)<2)
        {
            holes[i].acid++;
            shuddertimer=5;
            play_effect(1);
            return true;
        }
    }
    return false;
}

int timer;

struct GameLoop
{
    static void Init()
    {
        holes.clear();
        drops.clear();
        cursor=0;
        cursorframe=0;
        nodrill=true;
        reverb_on();
        play_music(2,0.8f); 
        position_mouse(40*8,15*8);
    }

    static bool Logic()
    {
           
        if (timer>0)
        {
            timer--;
            shuddertimer=2;
            if (timer==0)
                return false;
            
            return true;
            
        }
                                                                        
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        
        overdone = cursor==0 && mx<=28 && my>=53;
         
        int headoverlaps=-1;
        if ( mx>=29 && my >=29 && mx<29+23 && my<29+27)
        {
            int pix = getpixel(bmpHead2, mx-29, my-29);
            if (pix==makecol(255,0,0))
            {
                headoverlaps=0;
            }
            else if (pix==makecol(0,255,0))
            {
                headoverlaps=1;
            }
            else if (pix==makecol(0,0,255))
            {
                headoverlaps=2;
            }
            else if (pix==makecol(0,255,255))
            {
                headoverlaps=3;
            }
        }

        drillbone=headoverlaps>=0;

        if (actionpush)
        {
            if ( (mx>=2 && mx<= 2+24) && my>=2 && my <= 6+28 )
            {
                cursor= cursor==1 ? 0 : 1;
                nodrill=true;
                cursorframe=0;
                play_effect(1);
            }
               
            if (mx<=buffer->w-2 && mx>= buffer->w-2-24 && my>=2 && my <= 6+28)
            {
                cursor= cursor==2 ? 0 : 2;   
                nodrill=true;             
                cursorframe=0;
                play_effect(1);
            }
            
            if (overdone)
            {
                timer=2*updates_per_second;
                play_effect(0);
                play_effect(1);
            }
            
            drillprogress=0;
            drillratio=0;
            drilldepth=0;
        }    
        
        if (cursor==1)
        {
            if (actiondown)
            {
                if (!nodrill)
                {
                    freezecursor=true;                    
                    
                    drillprogress=min(drillprogress+1.0f/(2.0f*updates_per_second),1.0f);
                    if (drillprogress>=1.0f && drillbone)
                    {   
                        drillratio=min(drillratio+1.0f/(updates_per_second/2.0f),1.0f);
                        drilldepth=max(min(drilldepth+1.0f/(updates_per_second*1.5f),10.0f),1.3f);
                    }
                    
                    if (drillbone && drillprogress>=1)
                        makehole(mx,my,drilldepth);
                    
                    set_drill_params(true,drillprogress,drilldepth);
                }
            }
            else
            {
                nodrill=false;
                drillprogress=max(drillprogress-1.0f/(1.0f*updates_per_second),0.0f);
                drilldepth=min( max(0.0f,drilldepth-1.0f/(updates_per_second*2.0f)), drillprogress*10);
                
                if (freezecursor && (!drillbone||drillprogress<=0))
                {
                    position_mouse(mx*8,my*8);
                    mouse_b=0;
                    freezecursor=false;
                }

                if (drillprogress<=0)
                {
                    set_drill_params(false,drillprogress,drilldepth);
                }
                else
                {
                    set_drill_params(true,drillprogress,drilldepth);            
                }
            }
            
            static float drillframeupdate=0;
            drillframeupdate+=(float)drillprogress/3.0f;
            if (drillframeupdate>0.2f)
            {
                cursorframe = (cursorframe+1) %3;
                drillframeupdate=0;
            }
        }
        
        if (cursor==2)
        {
            if (actionpush)
            {
                if (!nodrill)
                {
                    cursorframe=1;
                    Pos d = (Pos){mx,my};
                    drops.push_back(d);
                }
            }
            else
            {
                nodrill=false;
                cursorframe=0;
            }
        }
        
        for (int i=0;i<drops.size();i++)
        {
            drops[i].y++;
            if (drops[i].y>=60 || dropletCheck(i))
            {
                drops.erase(drops.begin()+i);
                i--;
            }
        }
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (cursor!=1)
        {
                masked_blit(bmpDrill,buffer,0,0,4,4,21,28);
        }
           
        if ( (mx>=2 && mx<= 2+24) && my>=2 && my <= 6+28 )
        {
            rect(buffer,2,2,2+24,2+31,makecol(255,255,255));
            if (cursor!=1)
            {
                textout_centre_ex(buffer,font,"drill",80/2,15,makecol(255,255,255),makecol(0,0,0));
                textout_centre_ex(buffer,font,"drill",80/2+rand()%5-2,15+rand()%5-2,makecol(255,255,255),-1);
            }
        }
                
        if (cursor!=2)
        {
            masked_blit(bmpDropper,buffer,0,0,buffer->w-3-21+(21-7)/2,6,7,22);
        }
        
        if (mx<=buffer->w-2 && mx>= buffer->w-2-24 && my>=2 && my <= 6+28)
        {
            rect(buffer,buffer->w-2,2,buffer->w-2-24,2+31,makecol(255,255,255));
            if (cursor!=2)
            {
                textout_centre_ex(buffer,font,"acid",80/2,15,makecol(255,255,255),makecol(0,0,0));
                textout_centre_ex(buffer,font,"acid",80/2,15+rand()%5-2,makecol(255,255,255),-1);
            }
       
        }
        
        blit(bmpHead,buffer,0,0,29,29,23,27);
        
        for (int i=0;i<holes.size();i++)
        {
            masked_blit(bmpHoleMark,buffer,0,0,holes[i].x-1,holes[i].y-2,3,3);
        }
        
        if (cursor==0)
        {
            textout_ex(buffer,font,"done",0,53,makecol(255,255,255),-1);
            if (overdone)
                textout_ex(buffer,font,"done",0+rand()%5-2,53+rand()%5-2,makecol(255,255,255),-1);
        }
        
        for (int i=0;i<drops.size();i++)
        {
            putpixel(buffer,drops[i].x,drops[i].y,makecol(255,255,255));
        }

        switch (cursor)
        {
            case 0: 
                masked_blit(bmpCursor,buffer,0,0,mx,my,5,7);
                break;
            case 1: 
                int depthoffset=drilldepth;
                masked_blit(bmpDrill,buffer,21*cursorframe,28*(actiondown&&!nodrill),mx-4,my-28+depthoffset,21,28-depthoffset);
                break;
            case 2: 
                masked_blit(bmpDropper,buffer,7*cursorframe,0,mx-3,my-22,7,22);
                break;
        }

        if (drillratio>0&&drilldepth>0&&drilldepth<2)
        {
            for (int i=0;i<10;i++)
            {
                putpixel(buffer,rand()%80,rand()%60,makecol(0,0,0));
            }
        }
    }
};

struct IntroLoop
{
    static void Init()
    {
        play_music(0,0.8f); 
        timer = 6*updates_per_second;//fade in the last second
    }

    static bool Logic()
    {
        if (timer<0)
        {
            return false;
        }
            
        timer--;
        
        if ( (escpressed || key[KEY_X] || key[KEY_SPACE] || key[KEY_C] ) && timer>1*updates_per_second)
        {
            timer=1*updates_per_second;
        }
                                             
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
    //    if (!(( (timer<3*updates_per_second && (timer>2*updates_per_second)) || timer > 5*updates_per_second) && (timer%13==0 || timer%7==0)))
        {
            int offset1=0;
            int offset2=0;
  
            (timer%2==0 ? offset1 : offset2) = (timer%5==0 ? rand()%5 - 2 : 0);
                
            textout_centre_ex(buffer, font, "increpare", screenwidth/2, 10+offset1, makecol(255,255,255), -1);
            textout_centre_ex(buffer, font, "presents", screenwidth/2, 20+offset2, makecol(255,255,255), -1);
            textout_centre_ex(buffer, font, "increpare", screenwidth/2, 10, makecol(255,255,255), -1);
            textout_centre_ex(buffer, font, "presents", screenwidth/2, 20, makecol(255,255,255), -1);
        }
    }
};

template<int n, bool fademusic=false, int track=0>
struct Blackout
{
    static void Init()
    {
        timer = n*updates_per_second;//fade in the last second
    }

    static bool Logic()
    {
        if (timer<=0)
            return false;
            
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        
        timer--;
        
        if (fademusic)
            set_volume(track,0.4f * ((float)max(timer-n,0)) / ((float)n*(float)updates_per_second));
        
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
    }
};


struct GameOverLoop
{
    static void Init()
    {
        play_music(2,0.8f); 
    }

    static bool Logic()
    {            
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(255,255,255));
        textout_centre_ex(buffer,font,"THE",40,28,makecol(0,0,0),-1);
        textout_centre_ex(buffer,font,"END",40,38,makecol(0,0,0),-1);
        
    }
};

int selected;
int logicframe;
int movetimer;
bool uppressed=true;
bool downpressed=true;

struct TitleLoop
{
    static void Init()
    {
        if (!comingfromintro)
                play_music(0);

        comingfromintro=false;
        selected=0;
        logicframe=0;
        movetimer=0;
        uppressed=true;
        downpressed=true;
        timer=0;
    }
    
    
    static bool Logic()
    {
        logicframe=max(logicframe,logicframe+1);
        if (timer>0)
        {
            timer--;
            if (timer==0)
                return false;
            else
                return true;
        }
        
        if (movetimer>0)
            movetimer--;
            
        while (keypressed())
        {
            switch (readkey() >> 8) {
            case KEY_SPACE: 
            case KEY_X: 
            case KEY_C: 
                if (selected==0)
                {
                    shuddertimer=5;
                    timer=1*updates_per_second;
                    gamestate=GS_TitleBlackout;
                    play_effect(0);
                    play_effect(1);
                }
                else
                {
                    shuddertimer=5;
                    timer=1*updates_per_second;
                    gamestate=GS_Quit;
                    play_effect(0);
                    play_effect(1);
                }
            
            break;
            }
        }
        
        if (!downpressed)
        {
            if (key[KEY_DOWN])
            {
                selected=(selected+1)%2;
                play_effect(1);
                movetimer=updates_per_second/4;
            }
        }
        downpressed=key[KEY_DOWN];
        
        if (!uppressed)
        {
            if (key[KEY_UP])
            {
                selected=(selected+1)%2;
                play_effect(1);
                movetimer=updates_per_second/4;
            }
        }
        uppressed=key[KEY_UP];
        

        
        if (escpressed)
        {
            gamestate=GS_Quit;
            return false;
        }
                                                
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
    
        if ((logicframe/(rand()%11+1))%2==0)
            textout_ex(buffer,font,"brain",5+rand()%5-2,10+rand()%5-2,makecol(255,255,255),-1);
        
        textout_ex(buffer,font,"brain",5,10,makecol(255,255,255),-1);
        
        if ((logicframe/ (rand()%11+1))%2==0)
            textout_ex(buffer,font,"damage",31+rand()%5-2,18+rand()%5-2,makecol(255,255,255),-1);
          
        textout_ex(buffer,font,"damage",31,18,makecol(255,255,255),-1);
        
        int offsetx1=0;
        int offsety1=0;
        int offsetx2=0;
        int offsety2=0;
        if (movetimer>0 || timer>0)
        {
            offsetx1=rand()%5-2;
            offsety1=rand()%5-2;
            offsetx2=rand()%5-2;
            offsety2=rand()%5-2;
            if (gamestate!=GS_Quit)
                textout_ex(buffer,font,"start",15+offsetx1,40+offsety1,makecol(255,255,255),-1);
            if (gamestate!=GS_TitleBlackout)
                textout_ex(buffer,font,"quit",15+offsetx2,50+offsety2,makecol(255,255,255),-1);
        }
        
        
        if (gamestate!=GS_Quit)
            textout_ex(buffer,font,"start",15,40,makecol(255,255,255),-1);
        if (gamestate!=GS_TitleBlackout)
            textout_ex(buffer,font,"quit",15,50,makecol(255,255,255),-1);
        textout_ex(buffer,font,">",6,40+10*selected,makecol(255,255,255),makecol(0,0,0));
    }
};

int carframe;
bool drawman;
int drawmanx;

struct PreludeLoop
{
    static void Init()
    {
        selected=0;
        logicframe=0;
        movetimer=0;
        drawman=false;
        drawmanx=69;
        play_music(1,0.8f); 
        shuddertimer=2.5f*updates_per_second;
    }

    static bool Logic()
    {
        logicframe=max(logicframe,logicframe+1);
     
        if (drawmanx<=46)
        {
            if (logicframe>1100)
                return false;
        }
        else if (logicframe>16*updates_per_second)
        {
            drawman=true;
            if (logicframe%20==0)
            {
                carframe = (carframe+1)%5;   
                drawmanx--;
            }
            
        }
        else if (logicframe>15*updates_per_second)
        {
            if (logicframe%20==0)
                carframe = (carframe+1)%5;   
        }
        else if (logicframe>12*updates_per_second)
        {
            if (logicframe%10==0)
                carframe = (carframe+1)%5;            
        }
        else if (logicframe>2*updates_per_second)
        {
            if (logicframe%7==0)
                carframe = (carframe+1)%5;
        }
          
                                                                                      
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
    
        if (logicframe>2*updates_per_second)
        {
            blit(bmpCar,buffer,0,11*(carframe),0,29,80,11);
        
            if (drawman)
            {
                blit(bmpMan,buffer,((int)((((69-drawmanx)*3))/(69-46)))*5,0,drawmanx,30,5,8);
            }
        }
    }
};

const int DL1Count=7;

struct PreludeDialogueLoop
{
    static void Init()
    {
        logicframe=0;   
        timer=0;     
    }

    static bool Logic()
    {
        if (actionpush)
        {
            logicframe=max(logicframe,logicframe+1);
            if (logicframe>=DL1Count)
            {
                timer=updates_per_second*2;
            }
        }
        
        if (timer>0)
        {
            timer--;
            if (timer<=0)
                return false;
        }
                                                                                                                
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
                                                       
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        string s;
        string s2;
        bool speaker;
        
        switch (logicframe)
        {
            case 0:
                s="hey";
                speaker=0;
                break;
            case 1:
                s="want";
                s2="a ride?";
                speaker=0;
                break;
            case 2:
                s="";//-- face appears
                speaker=0;
                break;
            case 3:
                s="sure";
                speaker=1;
                break;
            case 4:
                s="hop in";
                speaker=0;
                break;
            case 5:
                s="back to";
                s2="mine?";
                speaker=0;
                break;
            case 6:
                s="sounds";
                s2="good";
                speaker=1;
                break;
        }
    
        rect(buffer,2, 2, 18, 23, makecol(255,255,255));
        blit(bmpFaces,buffer,0,0,3,4,15,19);

        rect(buffer,61, 36, 77, 57, makecol(255,255,255));
        blit(bmpFaces,buffer,15,0,62,38,15,19);
        
        if (speaker==0)
        {
            textout_ex(buffer, font, s.c_str(), 22,5, makecol(255,255,255), -1);
            textout_ex(buffer, font, s2.c_str(), 22,15, makecol(255,255,255), -1);
        }
        else
        {
            textout_right_ex(buffer, font, s.c_str(), 57,39, makecol(255,255,255), -1);
            textout_right_ex(buffer, font, s2.c_str(), 57,49, makecol(255,255,255), -1);
        
        }
    }
};

bool bigtext;
int facetoshow;
int convmode;
int timer2;

template<int n>
struct PostludeLoopIntro
{
    static void Init()
    {
        timer=0;    
        timer2=0;
        logicframe=0;        
        str="";    
        play_music(1,0.8f);
        bigtext=false;
        facetoshow=-1;
        convmode=-1;
    }

    static bool Logic()
    {

        if (timer2>0)
        {
            timer2--;
            if (timer2<=0)
                return false;
            else
                return true;
        }
        
        timer=max(timer,timer+1);
        textmode=rand()%3;
        randx=rand()%100-20;
        randy=rand()%80-10;
        
        
        if (timer<2*updates_per_second)
        {
            return true;
        }
        else if (timer<5*updates_per_second)
        {
            str="wake up";
        }
        else if (timer<6*updates_per_second)
        {
            str="";
        }
        else if (timer<9*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<12*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<14*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<16*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<17*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<18*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<18.5*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<19.5*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<20*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<20.25*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<20.5*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<20.5*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<20.6*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<20.7*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<20.8*updates_per_second)
        {
            bigtext=true;
            str="wake";
        }
        else if (timer<20.9*updates_per_second)
        {
            bigtext=true;
            str="up";
        }
        else if (timer<21.0*updates_per_second)
        {
            bigtext=false;
            str="";
        }
        else if (timer<26.0*updates_per_second)
        {
            facetoshow=n;
            str="";
            bigtext=false;
        }
        else 
        {
            return false;
        }
                                                                                                                
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
                                                       
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (str!="")
        {  
            switch (textmode)
            {
                case 0:
                    textout_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                break;
                case 1:
                    textout_centre_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                break;
                case 2:
                    textout_right_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                break;
            }
        }
        if (bigtext)
        {
            for (int i=0;i<20;i++)
            {
                switch (textmode)
                {
                    case 0:
                        textout_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                    break;
                    case 1:
                        textout_centre_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                    break;
                    case 2:
                        textout_right_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                    break;
                }  
                
                textmode=rand()%3;
                randx=rand()%100-20;
                randy=rand()%80-10;
                  
            }
        }
        else if (convmode>=0)
        {
            textout_centre_ex(buffer,font,str.c_str(),buffer->w/2,24,makecol(255,255,255),makecol(0,0,0));            
        }
        
        if (facetoshow>-1)
        {
            blit(bmpFaces,buffer,facetoshow*15,0,buffer->w/2-15/2,buffer->h/2-10,15,20);
            for (int i=0;i<holes.size();i++)
            {
                if (holes[i].y>=40)
                {
                    int targx=buffer->w/2-15/2 + 3+ (((float)holes[i].x-30.0f)/20.0f)*9.0f;
                    int targy= buffer->h/2-10+1 + (( (float)holes[i].y - 40.0f)/10.0f)*3.0f;
                    putpixel(buffer,targx,targy, makecol(255,255,255));
                }
            }
        }
        
    }
};

struct PostludeLoopBraindead
{
    static void Init()
    {
        timer=0;    
        timer2=0;
        logicframe=0;        
        str="";    
        bigtext=false;
        facetoshow=-1;
        convmode=0;
    }

    static bool Logic()
    {
        if (timer2>0)
        {
            timer2--;
            if (timer2<=0)
                return false;
            else
                return true;
        }
        
        timer=max(timer,timer+1);
        textmode=rand()%3;
        randx=rand()%100-20;
        randy=rand()%80-10;
        
              {
            bigtext=false;
            facetoshow=-1;
            if (convmode==-1)
                convmode=0;
            else
            {
                if (actionpush)
                {
                    convmode++;
                    if (convmode==9 && timer2==0)
                    {
                        timer2=2*updates_per_second;
                    }
                }
            }
                
            switch (convmode)
            {
                case 0:
                str="No.";
                break;
                case 1:
                str="That";
                break;
                case 2:
                str="Isn't";
                break;
                case 3:
                str="Right.";
                break;
                case 4:                
                str="That's";
                break;
                case 5:                
                str="Not";
                break;
                case 6:                
                str="What";
                break;
                case 7:                
                str="I";
                break;
                case 8:                
                str="Wanted.";
                break;
                case 9:
                str="";
                break;
                case 10:
                str="";
                break;
            }
        }
                                                                                                                
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
                                                       
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (str!="")
        {  
            switch (textmode)
            {
                case 0:
                    textout_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                break;
                case 1:
                    textout_centre_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                break;
                case 2:
                    textout_right_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                break;
            }
        }
        if (bigtext)
        {
            for (int i=0;i<20;i++)
            {
                switch (textmode)
                {
                    case 0:
                        textout_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                    break;
                    case 1:
                        textout_centre_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                    break;
                    case 2:
                        textout_right_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
                    break;
                }  
                
                textmode=rand()%3;
                randx=rand()%100-20;
                randy=rand()%80-10;
                  
            }
        }
        else if (convmode>=0)
        {
            textout_centre_ex(buffer,font,str.c_str(),buffer->w/2,24,makecol(255,255,255),makecol(0,0,0));            
        }
        
        if (facetoshow>-1)
        {
            blit(bmpFaces,buffer,facetoshow*15,0,buffer->w/2-15/2,buffer->h/2-10,15,20);
            for (int i=0;i<holes.size();i++)
            {
                if (holes[i].y>=40)
                {
                    int targx=buffer->w/2-15/2 + 3+ (((float)holes[i].x-30.0f)/20.0f)*9.0f;
                    int targy= buffer->h/2-10+1 + (( (float)holes[i].y - 40.0f)/10.0f)*3.0f;
                    putpixel(buffer,targx,targy, makecol(255,255,255));
                }
            }
        }
        
    }
};

int talker=0;
int subconv;
bool gamewon;
struct PostludeLoopQuestions
{
    static void Init()
    {
        timer=0;    
        timer2=0;
        logicframe=0;        
        str="";    
        bigtext=false;
        facetoshow=-1;
        convmode=0;
        talker=0;
        subconv=-1;
        gamewon=false;
    }

    static bool Logic()
    {
        if (timer2>0)
        {
            timer2--;
            if (timer2<=0)
                return false;
            else
                return true;
        }
        
        timer=max(timer,timer+1);
        textmode=rand()%3;
        randx=rand()%100-20;
        randy=rand()%80-10;
        
              {
            bigtext=false;
            facetoshow=-1;
            if (convmode==-1)
                convmode=0;
            else
            {
                if (actionpush)
                {
                    if (subconv>=0)
                        subconv++;
                    else    
                        convmode++;
                    

                }
            }
                
            switch (convmode)
            {
                case 0:
                talker=0;
                str="You were asleep.";
                break;
                case 1:
                talker=0;
                str="It's been over 24 hours since you've eaten";
                break;
                case 2:
                talker=0;
                str="Are you hungry?";
                break;
                case 3:
                    if (answers[0]==0)                        
                    {
                        talker=1;
                        str="Yes";
                    }
                    else
                    {
                        if (subconv==-1)
                            subconv=0;
                        switch (subconv)
                        {
                            case 0:
                            talker=1;
                            str="I ... I don't know.";
                            break;
                            case 1:
                            talker=0;
                            str="You don't know?";
                            break;
                            case 2:
                            talker=1;
                            str="I don't know! I ... I can't tell!";
                            break;
                            case 3:
                            talker=0;
                            str="That doesn't seem right";
                            break;
                            case 4:
                            talker=0;
                            str="We'll just have to try again, so.";
                            break;
                            case 5:
                            talker=1;
                            str="What ... what do you mean?";
                            break;
                            case 6:
                            return false;
                            break;
                        }
                    }
                break;
                case 4:
                talker=0;
                str="Do you want to eat?";
                break;
                case 5:                
                    if (answers[1]==1)                        
                    {
                        talker=1;
                        str="Yes";
                    }
                    else
                    {
                        if (subconv==-1)
                            subconv=0;
                        switch (subconv)
                        {
                            case 0:
                            talker=1;
                            str="Yes - I'm so hungry.";
                            break;
                            case 1:
                            talker=0;
                            str="That ... that isn't right.";
                            break;
                            case 2:
                            talker=1;
                            str="What do you mean?";
                            break;
                            case 3:
                            talker=0;
                            str="That isn't the right answer!";
                            break;
                            case 4:
                            talker=1;
                            str="I don't understand.";
                            break;
                            case 5:
                            talker=0;
                            str="You should not desire to act on your drives.";
                            break;
                            case 6:
                            talker=1;
                            str="I'm sorry ... I won't I promise!";
                            break;
                            case 7:
                            talker=0;
                            str="A man cannot recant his intents!";
                            break;
                            case 8:
                            talker=0;
                            str="They can only be excised by another.";
                            break;
                            case 9:
                            talker=0;
                            str="You are a failure.";
                            break;
                            case 10:
                            talker=1;
                            str="No ... please, no ...";
                            break;
                            case 11:
                            return false;
                            break;
                        }
                    }    
                    break;
                case 6:      
                talker=0;
                str="Good.";
                break;
                case 7:   
                talker=0;
                str="Here: eat this food.";
                break;
                case 8:                
                if (answers[2]==1)                        
                    {
                        talker=1;
                        str="Okay.";
                    }
                    else
                    {
                        if (subconv==-1)
                            subconv=0;
                        switch (subconv)
                        {
                            case 0:
                            talker=1;
                            str="I ... I don't want to.";
                            break;
                            case 1:
                            talker=0;
                            str="Hmm.  Why not?";
                            break;
                            case 2:
                            talker=1;
                            str="Please, just don't hurt me again.";
                            break;
                            case 3:
                            talker=0;
                            str="Why won't you do as I say?";
                            break;
                            case 4:
                            talker=1;
                            str="Please, just let me go.";
                            break;
                            case 5:
                            talker=0;
                            str="YOU PIECE OF SHIT.";
                            break;
                            case 6:
                            talker=1;
                            str="Let me go, please!";
                            break;
                            case 7:
                            talker=0;
                            str="I...this isn't working.";
                            break;
                            case 8:
                            talker=0;
                            str="We'll have to try again.";
                            break;
                            case 9:
                            return false;
                            break;
                        }
                    }
                break;
                case 9:
                talker=0;
                str="There's a good chap.";
                break;
                case 10:
                talker=0;
                str="I'll remove your restraints now, I think.";
                break;
                case 11:
                talker=1;
                str="Thank you.";
                break;
                case 12:
                talker=0;
                str="Could you get me a glass of water from the kitchen?";
                break;
                case 13:
                talker=1;
                str="Okay...";
                break;
                case 14:             
                if (answers[3]==0)                        
                    {
                        talker=0;
                        str="Thank you very much";
                    }
                    else
                    {
                        if (subconv==-1)
                            subconv=0;
                        switch (subconv)
                        {
                            case 0:
                            talker=1;
                            str="I ... I can't move.";
                            break;
                            case 1:
                            talker=0;
                            str="You can't move?";
                            break;
                            case 2:
                            talker=1;
                            str="Please...I can't move!";
                            break;
                            case 3:
                            talker=0;
                            str="Oh...";
                            break;
                            case 4:
                            talker=1;
                            str="What have you done to me?!";
                            break;
                            case 5:
                            talker=0;
                            str="Don't worry, I'll fix things.";
                            break;
                            case 9:
                            return false;
                            break;
                        }
                    }
                break;
                case 15:
                talker=0;
                str="You know what?";
                break;
                case 16:
                talker=1;
                str="What?";
                break;
                case 17:
                talker=0;
                str="I think...I think I've cracked it.";
                break;
                case 18:
                talker=0;
                str="I think ... you might be perfect ....";
                break;
                case 19:
                talker=1;
                str="Thank you";
                break;
                case 20:
                gamewon=true;
                return false;
                break;
            }
        }
                                                                                                                
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
                                                       
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (talker==1 && (rand()%2==0) )
        {
            blit(bmpFaces,buffer,2*15,0,buffer->w/2-15/2,buffer->h/2-10,15,20);
            for (int i=0;i<holes.size();i++)
            {
                if (holes[i].y>=40)
                {
                    int targx=buffer->w/2-15/2 + 3+ (((float)holes[i].x-30.0f)/20.0f)*9.0f;
                    int targy= buffer->h/2-10+1 + (( (float)holes[i].y - 40.0f)/10.0f)*3.0f;
                    putpixel(buffer,targx,targy, makecol(255,255,255));
                }
            }
        }
        
        
        if (str!="")
        {  
            wrapprint(str,0,0,10);
        }
        
        
    }
};


struct PostludeLoopLucid
{
    static void Init()
    {
        timer=0;    
        timer2=0;
        logicframe=0;        
        str="";    
        bigtext=false;
        facetoshow=3;
        convmode=0;
        talker=0;
        subconv=-1;
        gamewon=false;
    }

    static bool Logic()
    {
        if (timer2>0)
        {
            timer2--;
            if (timer2<=0)
                return false;
            else
                return true;
        }
        
        timer=max(timer,timer+1);
        textmode=rand()%3;
        randx=rand()%100-20;
        randy=rand()%80-10;
        
              {
            bigtext=false;
            facetoshow=-1;
            if (convmode==-1)
                convmode=0;
            else
            {
                if (actionpush)
                {
                    if (subconv>=0)
                        subconv++;
                    else    
                        convmode++;
                    

                }
            }
                
            switch (convmode)
            {
                case 0:
                talker=0;
                str="You were asleep.";
                break;
                case 1:
                talker=0;
                str="It's been over 24 hours since you ate";
                break;
                case 2:
                talker=0;
                str="Are you hungry?";
                break;
                case 3:
                talker=1;
                str="What ... what did you do to me?";
                break;
                case 4:
                talker=0;
                str="Don't worry, just answer my questions.";
                break;
                case 5:
                talker=1;
                str="Please ... let me go.";
                break;
                case 6:
                talker=0;
                if (!bParalysedButSane)                        
                    {
                        talker=0;
                        str="That's not the answer I was hoping to hear.";
                    }
                    else
                    {
                        if (subconv==-1)
                            subconv=0;
                        switch (subconv)
                        {
                            case 0:
                            talker=1;
                            str="Sure.  Allow me to take off your restraints.";
                            break;
                            case 1:
                            talker=1;
                            str="Th...thank you.";
                            break;
                            case 2:
                            talker=0;
                            str="You're free to go.";
                            break;
                            case 3:
                            talker=1;
                            str="I .... I can't move";
                            break;
                            case 4:
                            talker=1;
                            str="I can't move!";
                            break;
                            case 5:
                            talker=0;
                            str="I guess I'll have to remove you myself.";
                            break;
                            case 6:
                            return false;
                            break;
                        }
                    }
                break;
                case 7:
                talker=1;
                str="Don't hurt me any more!";
                break;
                case 8:
                return false;
                break;
            }
        }
                                                                                                                
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
                                                       
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (talker==1 && (rand()%2==0) )
        {
            blit(bmpFaces,buffer,2*15,0,buffer->w/2-15/2,buffer->h/2-10,15,20);
            for (int i=0;i<holes.size();i++)
            {
                if (holes[i].y>=40)
                {
                    int targx=buffer->w/2-15/2 + 3+ (((float)holes[i].x-30.0f)/20.0f)*9.0f;
                    int targy= buffer->h/2-10+1 + (( (float)holes[i].y - 40.0f)/10.0f)*3.0f;
                    putpixel(buffer,targx,targy, makecol(255,255,255));
                }
            }
        }
        
        
        if (str!="")
        {  
            wrapprint(str,0,0,10);
        }
        
        
    }
};

struct LoopbackLoop
{
    static void Init()
    {
        timer=0;    
        timer2=0;
        logicframe=0;        
        str="";    
        bigtext=false;
        facetoshow=-1;
        convmode=-1;
    }

    static bool Logic()
    {
        timer=max(timer,timer+1);
        //first wait for saw to be clicked, then do stuff
        shuddertimer=3;
                                                                                          
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        
        if (timer>8*updates_per_second)
            return false;
                                                                                                                                                            
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (timer<1*updates_per_second)
            blit(bmpCar,buffer,0,11*(carframe),0,29,80,11);
        else if (timer<2*updates_per_second)
        {
        }
        else if (timer<3*updates_per_second)
        {
            rect(buffer,2, 2, 18, 23, makecol(255,255,255));
            blit(bmpFaces,buffer,0,0,3,4,15,19);

            rect(buffer,61, 36, 77, 57, makecol(255,255,255));
            blit(bmpFaces,buffer,15,0,62,38,15,19);        
        }
        else if (timer<4*updates_per_second)
        {
        }
        else if (timer<5*updates_per_second)
        {            
            rect(buffer,2, 2, 18, 23, makecol(255,255,255));
            blit(bmpFaces,buffer,0,0,3,4,15,19);

            rect(buffer,61, 36, 77, 57, makecol(255,255,255));
            blit(bmpFaces,buffer,15,0,62,38,15,19);
            
            textout_ex(buffer, font, "have a", 22,5, makecol(255,255,255), -1);
            textout_ex(buffer, font, "beer", 22,15, makecol(255,255,255), -1);
            
        }
        else if (timer<6*updates_per_second)
        {
        }
        else if (timer<7*updates_per_second)
        {
            blit(bmpTable,buffer,0,23,10,3,58,23);
                
                
            blit(bmpFaces,buffer,15*8,1,0,40,14,20);
        }

    }
};



bool sawselected;
struct SawSelectLoop
{
    static void Init()
    {
        timer=0;    
        timer2=0;
        logicframe=0;        
        str="";    
        play_music(3,0.8f);
        bigtext=false;
        facetoshow=-1;
        convmode=-1;        
        sawselected=false;
        position_mouse(10,10);
    }

    static bool Logic()
    {
        if (timer>0)
        {
            timer--;
            if (timer<=0)
                return false;
        }
        //first wait for saw to be clicked, then do stuff
        sawselected= (mx>=buffer->w/2-bmpSaw->w/2-2 && mx<=buffer->w/2-bmpSaw->w/2+25+1
                && my >= buffer->h/2-bmpSaw->h/2-2 && my <= buffer->h/2-bmpSaw->h/2+9+1);
        
                                                                                            
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        
        if (actionpush && sawselected)
        {
            timer=1.0f*updates_per_second;
            play_effect(0);
            play_effect(1);
        }
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        blit(bmpSaw,buffer,0,0,buffer->w/2-bmpSaw->w/2,buffer->h/2-bmpSaw->h/2,25,9);
        
        if (sawselected)
        {
            if (timer==0 || rand()%2==0)
                rect(buffer, buffer->w/2-bmpSaw->w/2-2,buffer->h/2-bmpSaw->h/2-2,
                    buffer->w/2-bmpSaw->w/2+25+1,buffer->h/2-bmpSaw->h/2+9+1,makecol(255,255,255));
        }
        
        masked_blit(bmpCursor,buffer,0,0,mx,my,5,7);

    }
};






struct PostPreludeLoop
{
    static void Init()
    {
        logicframe=0;   
        timer=0;     
    }

    static bool Logic()
    {
        if (actionpush)
        {
            logicframe++;
            if (logicframe>=DL1Count)
            {
                timer=updates_per_second*2;
            }
        }
        
        if (timer>0)
        {
            timer--;
            if (timer<=0)
                return false;
        }
            
                                                                             
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }    
                                                                                                                  
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        string s;
        string s2;
        bool speaker;
        
        switch (logicframe)
        {
            case 0:
                s="please";
                speaker=0;
                break;
            case 1:
                s="have a";
                s2="beer";
                speaker=0;
                break;
            case 2:
                s="";//-- face appears
                speaker=0;
                break;
            case 3:
                s="okay";
                speaker=1;
                break;
            case 4:
                s="";
                speaker=1;
                break;
            case 5:
                s="thanks";
                speaker=1;
                break;
        }
    
        rect(buffer,2, 2, 18, 23, makecol(255,255,255));
        blit(bmpFaces,buffer,0,0,3,4,15,19);

        rect(buffer,61, 36, 77, 57, makecol(255,255,255));
        blit(bmpFaces,buffer,15,0,62,38,15,19);
        
        if (speaker==0)
        {
            textout_ex(buffer, font, s.c_str(), 22,5, makecol(255,255,255), -1);
            textout_ex(buffer, font, s2.c_str(), 22,15, makecol(255,255,255), -1);
        }
        else
        {
            textout_right_ex(buffer, font, s.c_str(), 57,39, makecol(255,255,255), -1);
            textout_right_ex(buffer, font, s2.c_str(), 57,49, makecol(255,255,255), -1);
        
        }
    }
};

bool showtable;
int tableframe;
int faceframe;


bool worried;

struct PostPrelude2Loop
{
    static void Init()
    {
        showtable=false;
        tableframe=0;
        logicframe=0;
        faceframe=-1;
        worried=false;
        convline=-1;
    }

    static bool Logic()
    {
        logicframe=max(logicframe,logicframe+1);
                                                                             
        if (escpressed)
        {
            gamestate=GS_Title;
            return false;
        }
        
        if (!worried)
        {
            switch (rand()%4)
            {
                case 0:
                str="urgh..";
                break;
                case 1:
                str="...my head...";
                break;
                case 2:
                str="where am I?";
                break;
                case 3:
                str="what did you...";
                break;
            }
        }
        else 
        {
            switch (rand()%4)
            {
                case 0:
                str="please...";
                break;
                case 1:
                str="...no...";
                break;
                case 2:
                str="let me...";
                break;
                case 3:
                str="...me go";
                break;
            }
        }
        
        textmode=rand()%3;
        randx=rand()%100-20;
        randy=rand()%80-10;
        
        if (timer>0)
        {
            timer--;
            if (timer==0)
            {
                return false;
            }
            return true;
        }
        
        if (logicframe<4*updates_per_second)
        {
            showtable=false;
            faceframe=-1;
        }
        else if (logicframe<8*updates_per_second)
        {
            showtable=(logicframe/(updates_per_second/2))%2==0;            
        }
        else if (logicframe<10*updates_per_second)
        {
            showtable=(logicframe/(updates_per_second/2))%2==0;            
        }
        else if (logicframe<12*updates_per_second)
        {        
            faceframe= (logicframe/(updates_per_second/2))%2==0 ? 8 : -1;
            showtable=(logicframe/(updates_per_second/2))%2==0;            
        }
        else if (logicframe<14*updates_per_second)
        {
            faceframe=7;
            showtable=(logicframe/(updates_per_second/2))%2==0;            
        }
        else
        {
            faceframe=7;
            showtable=(logicframe/(updates_per_second/2))%2==0;            
            if (convline==-1)
            {
                play_effect(1);            
                convline=0;
            }
                
            if (actionpush)
            {
                convline++;
                if (convline!=12 && convline!=16)
                    play_effect(1);
                
                if (convline==11)
                    play_effect(0);

                if (convline==5)
                {
                    worried=true;
                    tableframe=1;
                }
                else if (convline==16)
                {
                    timer=updates_per_second*15;
                }
            }
        }
        return true;
    }     

     
    static void Draw()
    {
        clear_to_color(buffer,makecol(0,0,0));
        
        if (timer==0 || timer>updates_per_second*10)
        {
            if (showtable)
                blit(bmpTable,buffer,0,23*tableframe,10,3,58,23);
                
                
            if (faceframe>-1)
                blit(bmpFaces,buffer,15*faceframe,1,0,40,14,20);
         }       
         
        switch (textmode)
        {
            case 0:
                textout_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
            break;
            case 1:
                textout_centre_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
            break;
            case 2:
                textout_right_ex(buffer,font,str.c_str(),randx,randy,makecol(255,255,255),-1);
            break;
        }
        
        if (convline>-1)
        {
            string sman;
            string sman2;
            switch (convline)
            {
                case 0:
                    sman="hush";
                    break;
                case 1:
                    sman="relax";
                    break;
                case 2:
                    sman="lie back";
                    sman2="down";
                    break;
                case 3:
                    sman="please";
                    break;
                case 4:
                    sman="lie back";
                    sman2="down";
                    break;
                case 5://frame changes
                    sman="";
                    break;
                case 6:
                    sman="there";
                    break;
                case 7:
                    sman="why...";
                    break;
                case 8:
                    sman="why";
                    sman2="must you";
                    break;
                case 9:
                    sman="disobey?";
                    break;
                case 10:
                    sman="why";
                    sman2="must you";
                    break;
                case 11:
                    sman="disobey?";
                    break;
                case 12:
                break;
                case 13:
                    sman="I will";
                    sman2="...";
                    break;
                case 14:
                    sman="I will";
                    sman2="be";
                    break;
                case 15:
                    sman="obeyed.";
                    break;

            }
                if (convline==11)
                {
                    textout_ex(buffer,font,sman.c_str(),16+rand()%5-2,41+rand()%5-2,makecol(255,255,255),-1);
                    textout_ex(buffer,font,sman2.c_str(),16+rand()%5-2,51+rand()%5-2,makecol(255,255,255),-1);          
                }
                
                textout_ex(buffer,font,sman.c_str(),16,41,makecol(255,255,255),-1);
                textout_ex(buffer,font,sman2.c_str(),16,51,makecol(255,255,255),-1);
            
        }
        
    }
};


//----------------------------
     
template<class LoopClass>
void DrawEverything()
{
    LoopClass::Draw();
    acquire_bitmap(screen);
    if (shuddertimer>0)
    {
        int r1=rand()%5;
        int r2=rand()%5;
        stretch_blit(buffer,screen,r1,r2,screenwidth-r1-r2,screenheight-r1-r2,0,0,resx,resy);
    }
    else
    {
        stretch_blit(buffer,screen,0,0,screenwidth,screenheight,0,0,resx,resy);
    }
    
    release_bitmap(screen);
}     

template<class LoopClass>
void LoopState()
{
        escdown=true;
        escpressed=false;
        clear_keybuf();

        actionpush=false;
        actiondown=true;
        
        rmx=resx/2;
        rmy=resy/2;
        
        freezecursor=false;
        shuddertimer=0;
        reverb_off();
        timer=0;
        logicframe=0;   

        LoopClass::Init();

        show_mouse(NULL);

        while(true)
        {
            while(ticks == 0)
            {
                rest(100 / updates_per_second);
            }
        
            while(ticks > 0)
            {
            
                    if (shuddertimer>0)
                        shuddertimer--;
                        
                    int old_ticks = ticks;
                    
                    if (!escdown && key[KEY_ESC])
                        escpressed=true;
                    
                    escdown = key[KEY_ESC];
                    
                    poll_mouse();
                    
                    if (freezecursor)
                    {
                     //   position_mouse(4*mx, 4*my);
                    }
                    else
                    {
                        mx=mouse_x/8;
                        my=mouse_y/8;
                    }
                    
                    actionpush=false;
                    if (!actiondown)
                    {   
                        if (key[KEY_SPACE]||key[KEY_X]||key[KEY_C] || (mouse_b & 1) || (mouse_b & 2))
                            actionpush=true;
                    }
                    actiondown=key[KEY_SPACE]||key[KEY_X]||key[KEY_C] || (mouse_b & 1) || (mouse_b & 2);
                    
                    
                    if (!LoopClass::Logic())
                    {
                        return;
                    }

                    ticks--;
                    if(old_ticks <= ticks)
                    break;
            }

            DrawEverything<LoopClass>();             
        }
        
        set_drill_params(false,drillprogress,drilldepth);
}

int main()
{
        srand(time(0));
        allegro_init();
        init_sound();
        
        install_keyboard();
        install_mouse();
        set_color_depth(32); 
        set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,resx,resy,0,0);

        buffer=create_bitmap(screenwidth,screenheight); 
        
        bmpFaces=load_bitmap("bmp/faces.bmp",NULL);
        bmpTable=load_bitmap("bmp/table.bmp",NULL);
        bmpDrill=load_bitmap("bmp/drill.bmp",NULL);
        bmpDropper=load_bitmap("bmp/dropper.bmp",NULL);
        bmpHead=load_bitmap("bmp/head.bmp",NULL);
        bmpCar=load_bitmap("bmp/car.bmp",NULL);
        bmpMan=load_bitmap("bmp/man.bmp",NULL);
        bmpCursor=load_bitmap("bmp/cursor.bmp",NULL);
        bmpHead2=load_bitmap("bmp/head2.bmp",NULL);
        bmpHoleMark=load_bitmap("bmp/holemark.bmp",NULL);
        bmpSaw=load_bitmap("bmp/saw.bmp",NULL);
        
        if (!bmpCar || !bmpMan || !bmpHead || !bmpDropper || !bmpDrill || !bmpTable || !bmpFaces )
            exit(1);
   
        #ifdef __APPLE__
        CGDisplayHideCursor(kCGDirectMainDisplay);
        #endif
     
        install_timer();

        LOCK_VARIABLE(ticks);
        LOCK_FUNCTION(ticker);
        install_int_ex(ticker, BPS_TO_TIMER(updates_per_second));

        gamestate=GS_Intro;
        
        while (true)
        {
            switch(gamestate)
            {
                case GS_Intro:
                    LoopState<IntroLoop>();
                    gamestate=GS_IntroBlackout;
                    break;
                case GS_IntroBlackout:
                    LoopState<Blackout<4> >();
                    gamestate=GS_Title;
                    break;
                case GS_Title:
                    LoopState<TitleLoop >();
                    if (gamestate==GS_Title)
                        gamestate=GS_TitleBlackout;
                    break;
                case GS_TitleBlackout:
                    LoopState<Blackout<6,true,0> >();
                    if (gamestate!=GS_TitleBlackout)
                        break;
                    LoopState<Blackout<3> >();
                    if (gamestate!=GS_TitleBlackout)
                        break;
                    if (gamestate==GS_TitleBlackout)
                        gamestate=GS_Prelude;
                    break;
                case GS_Prelude:
                    LoopState<PreludeLoop>();
                    if (gamestate==GS_Prelude)
                        gamestate=GS_PreludeDialogue;
                    break;
                case GS_PreludeDialogue:
                    LoopState<Blackout<4> >();
                    if (gamestate!=GS_PreludeDialogue)
                        break;
                    LoopState<PreludeDialogueLoop>();
                    if (gamestate==GS_PreludeDialogue)
                        gamestate=GS_PostPrelude;
                    break;
                case GS_PostPrelude:
                    LoopState<Blackout<6,true,1> >();
                    if (gamestate!=GS_PostPrelude)
                        break;
                    LoopState<Blackout<3> >();
                    if (gamestate!=GS_PostPrelude)
                        break;
                    LoopState<PostPreludeLoop>();
                    if (gamestate==GS_PostPrelude)
                        gamestate=GS_PostPrelude2;
                    break;
                case GS_PostPrelude2:
                    play_music(2,0.8f);
                    LoopState<Blackout<6> >();
                    if (gamestate!=GS_PostPrelude2)
                        break;
                    LoopState<PostPrelude2Loop>();
                    if (gamestate==GS_PostPrelude2)
                    {
                        LoopState<Blackout<3> >();
                        if (gamestate!=GS_PostPrelude2)
                            break;
                        LoopState<Blackout<2,true,2> >();
                        if (gamestate!=GS_PostPrelude2)
                            break;
                        gamestate=GS_Game;                        
                    }
                    break;
                case GS_Game:
                    LoopState<GameLoop>();
                    LoopState<Blackout<2,true,2> >();
                    if (gamestate!=GS_Game)
                        break;
                    if (gamestate==GS_Game)
                        gamestate=GS_Postlude;   
                    break;
                case GS_GameOver:
                    stop_music();
                    LoopState<Blackout<2,true,1> >();
                    if (gamestate!=GS_GameOver)
                        break;
                    LoopState<GameOverLoop >();                        
                    if (gamestate==GS_GameOver)
                        gamestate=GS_Title;   
                    break;
                case GS_Postlude:
                {
                    LoopState<Blackout<2> >();
                    if (gamestate!=GS_Postlude)
                            break;
                    calcBrainState();
                    gamewon=false;
                    switch (nBrainState)
                    {
                        case 0:
                        LoopState<PostludeLoopIntro<3> >();                    
                        if (gamestate!=GS_Postlude)
                                goto outswitch;
                        LoopState<PostludeLoopBraindead>(); 
                        break;
                        case 1:
                        LoopState<PostludeLoopIntro<2> >();                    
                        if (gamestate!=GS_Postlude)
                                goto outswitch;
                        LoopState<PostludeLoopQuestions>();
                        break;        
                        case 2:
                        LoopState<PostludeLoopIntro<2> >();                    
                        if (gamestate!=GS_Postlude)
                                goto outswitch;
                        LoopState<PostludeLoopLucid>();
                        break;                        
                    }
                    outswitch:
                    if (!gamewon && gamestate==GS_Postlude)
                    {
                        LoopState<Blackout<2,true,1> >();
                        gamestate=GS_LoopBack;
                    }
                    else if (gamewon && gamestate==GS_Postlude)
                        gamestate=GS_GameOver;
                    break;
                }
                case GS_LoopBack:
                    LoopState<Blackout<2> >();
                    if (gamestate!=GS_LoopBack)
                            break;
                    LoopState<SawSelectLoop>();
                    if (gamestate!=GS_LoopBack)
                            break;
                    LoopState<Blackout<1> >();
                    if (gamestate!=GS_LoopBack)
                            break;
                    LoopState<LoopbackLoop>();
                    if (gamestate!=GS_LoopBack)
                            break;
                    LoopState<Blackout<1,true,3> >();
                    if (gamestate!=GS_LoopBack)
                            break;
                    LoopState<Blackout<1,true,3> >();
                    if (gamestate==GS_LoopBack)
                    {
                        gamestate=GS_Game;
                    }
                    break;
                case GS_Quit:
                    goto doquit;
                    break;
            }
        }
        
        doquit:
        release_sound();
        return 0;
}
END_OF_MAIN()
