#include <iostream.h>
#include "stdio.h"
#include "musicback.h"
#include <math.h>

#include "fmod/fmod.h"
#include "fmod/fmod_errors.h"

#define DISTANCEFACTOR       1.0f    // Units per meter.  I.e feet would = 3.28.  centimeters would = 100.
            

	FMOD_SYSTEM* fmod_system;
    
	unsigned int      version;


const int SOUNDCOUNT=8;

char* sound_path[SOUNDCOUNT] =  {"sound/select.wav","sound/click.wav","sound/drillshallow.wav","sound/drilldeep.wav"};

FMOD_SOUND* fmod_sound[SOUNDCOUNT];

FMOD_CHANNEL    *sound_channel[SOUNDCOUNT];
	

const int MUSICCOUNT=4;
char* music_path[MUSICCOUNT] = {"sound/Title.ogg","sound/Prelude.ogg","sound/Game.ogg","sound/Interlude.ogg"};
FMOD_SOUND* fmod_music[MUSICCOUNT];

FMOD_CHANNEL    *music_channel[MUSICCOUNT];

float t=0;
void ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        exit(-1);
    }
}


void
init_sound()
{
                FMOD_RESULT      result;
              result = FMOD_System_Create(&fmod_system);
                  FMOD_SPEAKERMODE speakermode;
    FMOD_CAPS        caps;
        char             name[256];
    ERRCHECK(result);
    
    result = FMOD_System_GetVersion(fmod_system, &version);
    ERRCHECK(result);

    if (version < FMOD_VERSION)
    {
        printf("Error!  You are using an old version of FMOD %08x.  This program requires %08x\n", version, FMOD_VERSION);
    }

    result = FMOD_System_GetDriverCaps(fmod_system, 0, &caps, 0, 0, &speakermode);
    ERRCHECK(result);

    result = FMOD_System_SetSpeakerMode(fmod_system, speakermode);       /* Set the user selected speaker mode. */
    ERRCHECK(result);

    if (caps & FMOD_CAPS_HARDWARE_EMULATED)             /* The user has the 'Acceleration' slider set to off!  This is really bad for latency!. */
    {                                                   /* You might want to warn the user about this. */
        result = FMOD_System_SetDSPBufferSize(fmod_system, 1024, 10);
        ERRCHECK(result);
    }

    result = FMOD_System_GetDriverInfo(fmod_system, 0, name, 256, 0);
    ERRCHECK(result);

    if (strstr(name, "SigmaTel"))   /* Sigmatel sound devices crackle for some reason if the format is PCM 16bit.  PCM floating point output seems to solve it. */
    {
        result = FMOD_System_SetSoftwareFormat(fmod_system, 48000, FMOD_SOUND_FORMAT_PCMFLOAT, 0,0, FMOD_DSP_RESAMPLER_LINEAR);
        ERRCHECK(result);
    }

    result = FMOD_System_Init(fmod_system, 100, FMOD_INIT_NORMAL, 0);
    if (result == FMOD_ERR_OUTPUT_CREATEBUFFER)         /* Ok, the speaker mode selected isn't supported by this soundcard.  Switch it back to stereo... */
    {
        result = FMOD_System_SetSpeakerMode(fmod_system, FMOD_SPEAKERMODE_STEREO);
        ERRCHECK(result);
                
        result = FMOD_System_Init(fmod_system, 100, FMOD_INIT_NORMAL, 0);/* ... and re-init. */
        ERRCHECK(result);
    }

    /*
        Set the distance units. (meters/feet etc).
    */

            
                
    for (int i=0;i<SOUNDCOUNT;i++)
    {
        sound_channel[i]=0;
        FMOD_System_CreateSound(fmod_system,sound_path[i], FMOD_DEFAULT, 0, &(fmod_sound[i]));
        if (i==2||i==3) 
            FMOD_Sound_SetMode(fmod_sound[i], FMOD_LOOP_NORMAL );
        else
            FMOD_Sound_SetMode(fmod_sound[i], FMOD_LOOP_OFF );
    }

    for (int i=0;i<MUSICCOUNT;i++)
    {
        music_channel[i]=0;
        FMOD_System_CreateSound(fmod_system,music_path[i], FMOD_DEFAULT| FMOD_CREATESTREAM, 0, &(fmod_music[i]));
        FMOD_Sound_SetMode(fmod_music[i], FMOD_LOOP_NORMAL );
    }

}

void stop_music()
{
     for (int j=0;j<MUSICCOUNT;j++)
     {
        FMOD_Channel_Stop(music_channel[j]);        
     }

}

void stop_effects()
{
     for (int j=0;j<SOUNDCOUNT;j++)
    {
        FMOD_Channel_Stop(sound_channel[j]);  
     }
     
}

void set_drill_params(bool playing, float vol,float ratio)
{
    static bool curplaying=false;
    if (playing&&!curplaying)
    {
        FMOD_Channel_Stop( sound_channel[2] );
        FMOD_Channel_Stop( sound_channel[3] );
        FMOD_System_PlaySound(fmod_system, FMOD_CHANNEL_FREE, fmod_sound[2], 0, &sound_channel[2]);
        FMOD_System_PlaySound(fmod_system, FMOD_CHANNEL_FREE, fmod_sound[3], 0, &sound_channel[3]);
        curplaying=true;
    }
    else if (!playing&&curplaying)
    {
       	FMOD_Channel_Stop( sound_channel[2] );
       	FMOD_Channel_Stop( sound_channel[3] );
        curplaying=false;
        return;
    }
    if (playing)
    {
        FMOD_Channel_SetVolume(sound_channel[2],vol*(1-ratio));
        FMOD_Channel_SetVolume(sound_channel[3],vol*ratio);
       // FMOD_Channel_SetFrequency(sound_channel[2],0.7+0.3*vol);
       // FMOD_Channel_SetFrequency(sound_channel[3],0.7+0.3*vol);
    }
}


void reverb_on()
{
    FMOD_REVERB_PROPERTIES reverb=  FMOD_PRESET_DIZZY ;
    FMOD_System_SetReverbProperties(fmod_system, &reverb);
}

void reverb_off()
{
    FMOD_REVERB_PROPERTIES reverb= FMOD_PRESET_OFF  ;
    FMOD_System_SetReverbProperties(fmod_system, &reverb);
}

void set_volume(int channel, float f)
{
     FMOD_Channel_SetVolume(music_channel[channel],f); 
}

void play_music(int i, float vol)
{
     stop_music();
     FMOD_System_PlaySound(fmod_system, FMOD_CHANNEL_FREE, fmod_music[i], 0, &music_channel[i]);   
     FMOD_Channel_SetVolume(music_channel[i],vol);     
}

void play_effect(int i)
{     
   	FMOD_Channel_Stop( sound_channel[i] );
    FMOD_System_PlaySound(fmod_system, FMOD_CHANNEL_FREE, fmod_sound[i], 0, &sound_channel[i]);
    FMOD_Channel_SetVolume(sound_channel[i],0.2f);
}
     void
release_sound()
{
    for (int i=0;i<SOUNDCOUNT;i++)     
        FMOD_Sound_Release(fmod_sound[i]);
        
    for (int i=0;i<MUSICCOUNT;i++)     
        FMOD_Sound_Release(fmod_music[i]);
        
    FMOD_System_Close(fmod_system);
    FMOD_System_Release(fmod_system);
}

